package com.mcwl.resource.handler.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.ModelImageLike;
import com.mcwl.resource.domain.SysAdvice;
import com.mcwl.resource.handler.IMessageHandler;
import com.mcwl.resource.mapper.ModelImageLikeMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.mapper.WorkFlowLikeMapper;
import com.mcwl.resource.mapper.WorkFlowMapper;
import com.mcwl.resource.service.ISysAdviceService;
import com.mcwl.system.domain.enums.AdviceEnum;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

@Slf4j
@Component
@IMessageHandler.Topic(value = "imageLike/#", qos = 1)
@RequiredArgsConstructor
public class ImageLikeHandler implements IMessageHandler {


    private final ModelImageMapper modelImageMapper;

    private final ModelImageLikeMapper modelImageLikeMapper;

    private final ISysUserService userService;

    private final ISysAdviceService adviceService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleMessage(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        log.info("图片点赞: {}", payload);

        JSON parse = JSONUtil.parse(payload);
        Long commentId = Long.parseLong(parse.getByPath("commentId").toString());
        Long userId = Long.parseLong(parse.getByPath("userId").toString());

        this.like(userId, commentId);
    }

    private void like(Long userId, Long imageId) {
        ModelImage modelImage = modelImageMapper.selectById(imageId);
        if (Objects.isNull(modelImage)) {
            throw new ServiceException("该图片不存在或已下架");
        }
        ModelImageLike modelImageLike = modelImageLikeMapper.getLikeImage(userId, imageId);
        if (Objects.nonNull(modelImageLike)) {
            if (Objects.equals(modelImageLike.getDelFlag(), "0")) {
                modelImageLike.setDelFlag("2");
                int likeNum = modelImage.getLikeNum() - 1;
                likeNum = Math.max(likeNum, 0);
                modelImage.setLikeNum(likeNum);
            } else {
                modelImageLike.setDelFlag("0");
                modelImage.setLikeNum(modelImage.getLikeNum() + 1);
                this.addLikeAdvice(modelImage, userId);
            }
            // 更新点赞记录
            modelImageLikeMapper.updateDelFlagById(modelImageLike);
            // 更新图片点赞数
            modelImageMapper.updateById(modelImage);
            return;
        }

        // 添加点赞记录
        modelImageLike = new ModelImageLike();
        modelImageLike.setUserId(userId);
        modelImageLike.setModelImageId(imageId);
        modelImageLikeMapper.insert(modelImageLike);

        // 更新图片点赞数
        modelImage.setLikeNum(modelImage.getLikeNum() + 1);
        modelImageMapper.updateById(modelImage);

        this.addLikeAdvice(modelImage, userId);
    }


    private void addLikeAdvice(ModelImage modelImage, Long userId) {

        Long receiverUserId = modelImage.getUserId();

        if (Objects.equals(userId, receiverUserId)) {
            return;
        }

        SysUser sysUser = userService.selectUserById(userId);


        String content = StringUtils.format("恭喜！{}点赞了您的图片：{}",
                sysUser.getNickName(), modelImage.getTitle());

        SysAdvice sysAdvice = new SysAdvice();
        sysAdvice.setSenderId(userId);
        sysAdvice.setReceiverId(receiverUserId);
        sysAdvice.setContent(content);
        sysAdvice.setProductId(modelImage.getId());
        sysAdvice.setProductType(2);
        sysAdvice.setIsRead(0);
        sysAdvice.setType(AdviceEnum.LIKE_REMIND);
        adviceService.save(sysAdvice);


    }


}