package com.mcwl.resource.handler.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.SysAdvice;
import com.mcwl.resource.domain.WorkFlowComment;
import com.mcwl.resource.domain.WorkFlowCommentLike;
import com.mcwl.resource.handler.IMessageHandler;
import com.mcwl.resource.mapper.WorkFlowCommentLikeMapper;
import com.mcwl.resource.mapper.WorkFlowCommentMapper;
import com.mcwl.resource.service.ISysAdviceService;
import com.mcwl.system.domain.enums.AdviceEnum;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

@Slf4j
@Component
@IMessageHandler.Topic(value = "workFlowCommentLike/#", qos = 1)
@RequiredArgsConstructor
public class WorkFlowCommentLikeHandler implements IMessageHandler {


    private final WorkFlowCommentMapper workFlowCommentMapper;


    private final WorkFlowCommentLikeMapper workFlowCommentLikeMapper;

    private final ISysUserService userService;

    private final ISysAdviceService adviceService;



    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleMessage(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        log.info("工作流评论点赞: {} - {}", topic, payload);

        JSON parse = JSONUtil.parse(payload);
        Long commentId = Long.parseLong(parse.getByPath("commentId").toString());
        Long userId = Long.parseLong(parse.getByPath("userId").toString());


        this.like(userId, commentId);
    }

    private void like(Long userId, Long commentId) {
        WorkFlowComment workFlowComment = workFlowCommentMapper.selectById(commentId);
        if (Objects.isNull(workFlowComment)) {
            throw new ServiceException("该评论不存在");
        }
        WorkFlowCommentLike workFlowCommentLike = workFlowCommentLikeMapper.getLikeComment(userId, commentId);
        if (Objects.nonNull(workFlowCommentLike)) {
            if (Objects.equals(workFlowCommentLike.getDelFlag(), "0")) {
                workFlowCommentLike.setDelFlag("2");
                int likeNum = workFlowComment.getLikeNum() - 1;
                likeNum = Math.max(likeNum, 0);
                workFlowComment.setLikeNum(likeNum);
            } else {
                workFlowCommentLike.setDelFlag("0");
                workFlowComment.setLikeNum(workFlowComment.getLikeNum() + 1);
                this.addLikeAdvice(workFlowComment, userId);
            }
            // 更新点赞记录
            workFlowCommentLikeMapper.updateDelFlagById(workFlowCommentLike);
            // 更新图片评论点赞数
            workFlowCommentMapper.updateById(workFlowComment);
            return;
        }

        // 添加点赞记录
        workFlowCommentLike = new WorkFlowCommentLike();
        workFlowCommentLike.setUserId(userId);
        workFlowCommentLike.setWorkFlowCommentId(commentId);
        workFlowCommentLikeMapper.insert(workFlowCommentLike);

        // 更新模型点赞数
        workFlowComment.setLikeNum(workFlowComment.getLikeNum() + 1);
        workFlowCommentMapper.updateById(workFlowComment);

        this.addLikeAdvice(workFlowComment, userId);
    }


    private void addLikeAdvice(WorkFlowComment workFlowComment, Long userId) {

        Long receiverUserId = workFlowComment.getUserId();

        if (Objects.equals(userId, receiverUserId)) {
            return;
        }
        SysUser sysUser = userService.selectUserById(userId);

        String content = StringUtils.format("恭喜！{}点赞了您的评论",
                sysUser.getNickName());

        SysAdvice sysAdvice = new SysAdvice();
        sysAdvice.setSenderId(userId);
        sysAdvice.setReceiverId(receiverUserId);
        sysAdvice.setContent(content);
        sysAdvice.setProductId(workFlowComment.getWorkFlowId());
        sysAdvice.setProductType(1);
        sysAdvice.setCommentId(workFlowComment.getId());
        sysAdvice.setIsRead(0);
        sysAdvice.setType(AdviceEnum.LIKE_REMIND);
        adviceService.save(sysAdvice);


    }



}