package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.constant.QueueConstants;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.WorkFlow;
import com.mcwl.resource.domain.WorkFlowLike;
import com.mcwl.resource.domain.vo.WorkFlowLikeVo;
import com.mcwl.resource.mapper.WorkFlowLikeMapper;
import com.mcwl.resource.mapper.WorkFlowMapper;
import com.mcwl.resource.service.WorkFlowLikeService;
import com.mcwl.resource.util.MqttTemplate;
import com.mcwl.system.service.ISysUserService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 工作流点赞
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.service.impl
 * @Filename：ModelLikeServiceImpl
 * @Description TODO
 * @Date：2025/1/12 12:05
 */
@Service
public class WorkFlowLikeServiceImpl extends ServiceImpl<WorkFlowLikeMapper, WorkFlowLike> implements WorkFlowLikeService {


    @Autowired
    private WorkFlowMapper workFlowMapper;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private MqttTemplate mqttTemplate;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public void like(Long modelId) {
        Map<String, Long> map = new HashMap<>();
        map.put("userId", SecurityUtils.getUserId());
        map.put("commentId", modelId);
//        mqttTemplate.publish("workFlowLike", JSONUtil.toJsonStr(map),0);
        rabbitTemplate.convertAndSend(QueueConstants.WORK_FLOW_LIKE_QUEUE, map);
    }

    @Override
    public TableDataInfo listByPage(PageDomain pageDomain) {

        Page<WorkFlowLike> page = initPage(pageDomain);


        baseMapper.selectPage(page, new LambdaQueryWrapper<WorkFlowLike>()
                .eq(WorkFlowLike::getUserId, SecurityUtils.getUserId()));

        List<WorkFlowLike> workFlowLikeList = page.getRecords();
        List<WorkFlowLikeVo> modelLikeVoList = new ArrayList<>();
        for (WorkFlowLike workFlowLike : workFlowLikeList) {
            WorkFlowLikeVo workFlowLikeVo = new WorkFlowLikeVo();
            BeanUtil.copyProperties(workFlowLike, workFlowLikeVo);

            // 获取用户信息
            SysUser sysUser = sysUserService.selectUserById(workFlowLike.getUserId());
            if (Objects.isNull(sysUser)) {
                page.setTotal(page.getTotal() - 1);
                continue;
            }
            workFlowLikeVo.setUserName(sysUser.getNickName());
            workFlowLikeVo.setUserAvatar(sysUser.getAvatar());

            // 获取工作流信息
            WorkFlow workFlow = workFlowMapper.selectById(workFlowLike.getWorkFlowId());
            if (Objects.isNull(workFlow)) {
                page.setTotal(page.getTotal() - 1);
                continue;
            }
            workFlowLikeVo.setDownloadNumber(workFlow.getDownloadNumber());
            workFlowLikeVo.setCoverPath(workFlow.getCoverPath());
            workFlowLikeVo.setLikeNum(workFlow.getLikeNum());

            modelLikeVoList.add(workFlowLikeVo);
        }


        TableDataInfo tableDataInfo = new TableDataInfo();
        tableDataInfo.setRows(modelLikeVoList);
        tableDataInfo.setTotal(page.getTotal());
        tableDataInfo.setCode(HttpStatus.SUCCESS);
        tableDataInfo.setMsg("查询成功");

        return tableDataInfo;
    }

    @Override
    public WorkFlowLike selectWorkFlowLikeById(Long userId, Long id) {

        LambdaQueryWrapper<WorkFlowLike> workFlowLikeLambdaQueryWrapper = new LambdaQueryWrapper<>();
        workFlowLikeLambdaQueryWrapper.eq(WorkFlowLike::getWorkFlowId,id);
        workFlowLikeLambdaQueryWrapper.eq(WorkFlowLike::getUserId,userId);

        return baseMapper.selectOne(workFlowLikeLambdaQueryWrapper);
    }





    private Page<WorkFlowLike> initPage(PageDomain pageDomain) {
        Page<WorkFlowLike> page = new Page<>(pageDomain.getPageNum(), pageDomain.getPageSize());
        if (StringUtils.isBlank(pageDomain.getOrderByColumn())) {
            pageDomain.setOrderByColumn("create_time");
        }
        OrderItem orderItem = OrderItem.desc(pageDomain.getOrderByColumn());
        page.addOrder(orderItem);

        return page;
    }

}
